<?php
/**
 * @copyright   Copyright (c) http://www.manadev.com
 * @license     http://www.manadev.com/license  Proprietary License
 */
namespace Manadev\Core\Preferences;

use Magento\Config\Model\Config\Structure\Converter;
use Magento\Framework\Config\FileIterator;
use Magento\Framework\View\TemplateEngine\Xhtml\CompilerInterface;
use Manadev\Core\Auth;

class SystemConfigReader extends \Magento\Config\Model\Config\Structure\Reader {
    /**
     * @var Auth
     */
    private $auth;

    /**
     * Constructor
     *
     * @param \Magento\Framework\Config\FileResolverInterface $fileResolver
     * @param Converter $converter
     * @param \Magento\Config\Model\Config\SchemaLocator $schemaLocator
     * @param \Magento\Framework\Config\ValidationStateInterface $validationState
     * @param CompilerInterface $compiler
     * @param Auth $auth
     * @param string $fileName
     * @param array $idAttributes
     * @param string $domDocumentClass
     * @param string $defaultScope
     */
    public function __construct(
        \Magento\Framework\Config\FileResolverInterface $fileResolver,
        Converter $converter,
        \Magento\Config\Model\Config\SchemaLocator $schemaLocator,
        \Magento\Framework\Config\ValidationStateInterface $validationState,
        CompilerInterface $compiler,
        Auth $auth,
        $fileName = 'system.xml',
        $idAttributes = [],
        $domDocumentClass = 'Magento\Framework\Config\Dom',
        $defaultScope = 'global'
    ) {
        $this->auth = $auth;
        parent::__construct($fileResolver, $converter, $schemaLocator, $validationState, $compiler, $fileName, $idAttributes, $domDocumentClass, $defaultScope); // TODO: Change the autogenerated stub
    }

    /**
     * Read configuration files
     *
     * @param FileIterator $fileList
     * @return array
     * @throws LocalizedException
     */
    protected function _readFiles($fileList) {
        if($fileList instanceof FileIterator) {
            $fileList = $fileList->toArray();
        }
        foreach($fileList as $file => $content) {
            $parts = explode("/",$file);
            $module = implode("_", [$parts[count($parts) - 5], $parts[count($parts) - 4]]);
            if(!$this->auth->isModuleEnabled($module, 0)) {
                unset($fileList[$file]);
            }
        }

        return parent::_readFiles($fileList);
    }
}